@@ -112,6 +112,161 @@ class StarRocks(MySQL):
 
         TRANSFORMS.pop(exp.DateTrunc)
 
+        # https://docs.starrocks.io/docs/sql-reference/sql-statements/keywords/#reserved-keywords
+        RESERVED_KEYWORDS = {
+            "add",
+            "all",
+            "alter",
+            "analyze",
+            "and",
+            "array",
+            "as",
+            "asc",
+            "between",
+            "bigint",
+            "bitmap",
+            "both",
+            "by",
+            "case",
+            "char",
+            "character",
+            "check",
+            "collate",
+            "column",
+            "compaction",
+            "convert",
+            "create",
+            "cross",
+            "cube",
+            "current_date",
+            "current_role",
+            "current_time",
+            "current_timestamp",
+            "current_user",
+            "database",
+            "databases",
+            "decimal",
+            "decimalv2",
+            "decimal32",
+            "decimal64",
+            "decimal128",
+            "default",
+            "deferred",
+            "delete",
+            "dense_rank",
+            "desc",
+            "describe",
+            "distinct",
+            "double",
+            "drop",
+            "dual",
+            "else",
+            "except",
+            "exists",
+            "explain",
+            "false",
+            "first_value",
+            "float",
+            "for",
+            "force",
+            "from",
+            "full",
+            "function",
+            "grant",
+            "group",
+            "grouping",
+            "grouping_id",
+            "groups",
+            "having",
+            "hll",
+            "host",
+            "if",
+            "ignore",
+            "immediate",
+            "in",
+            "index",
+            "infile",
+            "inner",
+            "insert",
+            "int",
+            "integer",
+            "intersect",
+            "into",
+            "is",
+            "join",
+            "json",
+            "key",
+            "keys",
+            "kill",
+            "lag",
+            "largeint",
+            "last_value",
+            "lateral",
+            "lead",
+            "left",
+            "like",
+            "limit",
+            "load",
+            "localtime",
+            "localtimestamp",
+            "maxvalue",
+            "minus",
+            "mod",
+            "not",
+            "ntile",
+            "null",
+            "on",
+            "or",
+            "order",
+            "outer",
+            "outfile",
+            "over",
+            "partition",
+            "percentile",
+            "primary",
+            "procedure",
+            "qualify",
+            "range",
+            "rank",
+            "read",
+            "regexp",
+            "release",
+            "rename",
+            "replace",
+            "revoke",
+            "right",
+            "rlike",
+            "row",
+            "row_number",
+            "rows",
+            "schema",
+            "schemas",
+            "select",
+            "set",
+            "set_var",
+            "show",
+            "smallint",
+            "system",
+            "table",
+            "terminated",
+            "text",
+            "then",
+            "tinyint",
+            "to",
+            "true",
+            "union",
+            "unique",
+            "unsigned",
+            "update",
+            "use",
+            "using",
+            "values",
+            "varchar",
+            "when",
+            "where",
+            "with",
+        }
+
         def create_sql(self, expression: exp.Create) -> str:
             # Starrocks' primary key is defined outside of the schema, so we need to move it there
             schema = expression.this
